﻿$here = Split-Path -Parent $MyInvocation.MyCommand.Path
. "$here\Service.ps1"


# Windows ファイルウォールサービスが起動しているかを返す
function Test-FirewallService-Started {
    return Test-ServiceRunning -serviceName "MpsSvc"
}


# ファイアウォールが有効かどうかチェックする
# $true:有効/$false:無効
function Test-Firewall-Enabled {
    param (
        $firewallProfiles
    )

    # NICに結びついたFWルールがあれば有効を返す
    if ($null -ne $firewallProfiles) {
        return $true
    }

    return $false
}

Function Get-EnabledNetWorkFirewallProfiles() {
    # NICごとに属しているプロファイルを取得する
    $netConnectionProfiles = Get-NetConnectionProfile | Select-Object -ExpandProperty NetworkCategory

    $fw_profiles = @()
    # ファイルウォールプロファイル名に変換
    foreach ($profile in $netConnectionProfiles) {
        $fw_profile_name = $profile
        if ($profile -eq "DomainAuthenticated") {
            $fw_profile_name = "Domain"
        }

        $firewallProfile = Get-NetFirewallProfile | Where-Object { $_.Name -eq $fw_profile_name }
        # プロファイルが有効ならばFWありを返す
        if ($firewallProfile.Enabled) {
            if ($fw_profile_name -notin $fw_profiles) {
                $fw_profiles += $fw_profile_name
            }
        }
    }
    return $fw_profiles
}

# ファイアウォールルールクラス
class FirewallRule {
    $inboundRules = @{}

    FirewallRule($profiles) {
        # 受信ルール - ルールが有効なもの
        $rules = Get-NetFirewallRule -Direction Inbound
        foreach ($rule in $rules) {
            if ($rule.Profile -eq "Any") {
                $this.inboundRules[$rule.Name] = $rule
            } else {
                foreach ($profile in $profiles) {
                    if ($rule.Profile -match $profile) {
                        $this.inboundRules[$rule.Name] = $rule
                    }
                }
            }
        }
    }

    # ファイルウォールルールが有効かどうか
    # ruleNme名は表示名ではなく、Get-NetFirewallRule で取得されるnameを指定する
    [bool] EnabledRule([string]$ruleName) {
        $rule = $this.inboundRules[$ruleName]
        if ($rule) {
            #受信ルールなので有効な受信OKを返す
            if ($rule.Enabled -eq $True) {
                return $true
            } else {
                return $false
            }
        }
        # ルールがないので受信OKと判断
        return $true
    }

    # ファイルウォールルールが有効かどうか
    # ruleNme名は表示名ではなく、Get-NetFirewallRule で取得されるnameを指定する
    # 一部「xxxx-NoScope」のようなルール名が存在するので前方一致で比較する関数
    [bool] EnabledRuleStartsWith([string]$ruleName) {
        foreach ($key in $this.inboundRules.Keys) {
            if ($key.StartsWith($ruleName)) {
                #受信ルールなので有効な受信OKを返す
                $rule = $this.inboundRules[$key]
                if ($rule.Enabled -eq $True) {
                    return $true
                } else {
                    return $false
                }
            }
        }
        # ルールがないので受信OKと判断
        return $true
    }
}

# SMB(TCP445)ポート関係のファイアウォールルールが有効かどうかチェック
function Test-NetFirewallRule-SMB {
    param (
        [Parameter(Mandatory)][FirewallRule]$firewallRuleObject
    )

    $checkRules = @(
        "Netlogon-NamedPipe-In", #Netlogon サービス (NP 受信)
        "FPS-SMB-In-TCP", #ファイルとプリンターの共有 (SMB 受信)
        "RemoteEventLogSvc-NP-In-TCP", #リモート イベントのログ管理 (NP 受信)
        "RemoteSvcAdmin-NP-In-TCP" #リモート サービス管理 (NP 受信)
    )

    foreach ($rule in $checkRules) {
        if ($firewallRuleObject.EnabledRuleStartsWith($rule)) {
            return $true
        }
    }

    return $false
}

# RPC待ち受け(TCP135)ポート関係のファイアウォールルールが有効かどうかチェック
function Test-NetFirewallRule-RPC {
    param (
        [Parameter(Mandatory)][FirewallRule]$firewallRuleObject
    )

    $checkRules = @(
        #cspell:disable-next-line
        "ComPlusNetworkAccess-DCOM-In" #COM+ ネットワーク アクセス (DCOM-受信)
    )

    foreach ($rule in $checkRules) {
        if ($firewallRuleObject.EnabledRule($rule)) {
            return $true
        }
    }

    return $false
}

# RPC動的ポートのファイアウォールルールが有効かどうかチェック
function Test-NetFirewallRule-RPCDynamic {
    param (
        [Parameter(Mandatory)][FirewallRule]$firewallRuleObject
    )

    $checkRules = @(
        "RemoteSvcAdmin-RPCSS-In-TCP", #リモート サービス管理 (RPC-EPMAP)
        "RemoteEventLogSvc-In-TCP" #リモート イベントのログ管理 (RPC)
    )

    #両方有効でない場合はエラーを返す
    foreach ($rule in $checkRules) {
        # クライアントOSの場合「xxxx-NoScope」のようなルール名が存在するので前方一致で比較する関数
        if (!$firewallRuleObject.EnabledRuleStartsWith($rule)) {
            return $false
        }
    }

    return $true
}
