﻿Param(
    # domain 255, \ 1, user 255
    [parameter(mandatory = $true)][ValidateLength(1, 512)][string]$account
)


#共通処理の読み込み
. ..\common\Common.ps1
. ..\common\User.ps1
. ..\common\Uac.ps1
. ..\common\Service.ps1
. ..\common\System.ps1
. ..\common\Firewall.ps1

# ユーザー周りのチェック
function Test-UserRelationshipSettings {
    param(
        [Parameter(Mandatory)][string]$account
    )

    $user_account = Get-UserAccountInfo $account

    if ($user_account) {
        #監視アカウントがコンピューターの、Administratorsグループに属しているか確認する。
        Write-Result -Item "${account}ユーザーがAdministratorsグループに属している" -IsSuccess (Test-AdministratorGroupMembership $user_account)
        # ユーザーが有効化されているか確認する。
        if ($user_account.Disabled) {
            Write-Result -Item "${account}ユーザーが有効化されている" -IsSuccess $false
        }
        return $user_account
    } else {
        Write-Result -Item "${account}ユーザーは存在しません。" -IsSuccess $false
    }
    return $null
}

# FW関係のチェック
function Test-Windows-FireWall {
    if ($True -eq (Test-FirewallService-Started)) {
        $firewallProfiles = Get-EnabledNetWorkFirewallProfiles
        $is_fw_enabled = Test-Firewall-Enabled $firewallProfiles
        if ($is_fw_enabled) {
            $fw_rule = [FirewallRule]::new($firewallProfiles)
            Write-Result -Item "TCP445のルールが有効か確認" -IsSuccess (Test-NetFirewallRule-SMB -firewallRuleObject $fw_rule)
            Write-Result -Item "TCP135のルールが有効か確認" -IsSuccess (Test-NetFirewallRule-RPC -firewallRuleObject $fw_rule)
            Write-Result -Item "TCP動的ポート(TCP49152-65535)のルールが有効か確認" -IsSuccess (Test-NetFirewallRule-RPCDynamic -firewallRuleObject $fw_rule)
        } else {
            Write-Host "Windows ファイアウォールのプロファイルが無効化されています。"
            Write-Host "Windows ファイアウォールのルールチェックをスキップします。"
        }
    } else {
        Write-Host "Windows ファイアウォールサービスが起動していません。"
        Write-Host "Windows ファイアウォールのルールチェックをスキップします。"
    }

}

###############################################
# ここからメイン

$is_admin = TestAdministrator
if ( -not($true -eq $is_admin ) ) {
    Write-Error "管理者として実行してください。"
    Exit
}

# ユーザー周りの確認
$user_account = Test-UserRelationshipSettings -account $account

#ユーザーアカウント制御(UAC)が無効化されているか確認する。
Write-Result -Item "UAC無効化確認" -IsSuccess (-Not(Get-UACEnabled))

# FWルールの確認
Test-Windows-FireWall

#Remote Registryサービスの確認
$remote_reg_svc = "RemoteRegistry"
Write-Result -Item "Remote Registryサービスが自動起動になっているか確認" -IsSuccess (Test-ServiceAutoStartup $remote_reg_svc)
Write-Result -Item "Remote Registryサービスが起動しているか確認" -IsSuccess (Test-ServiceRunning $remote_reg_svc)

#Guest認証設定確認
Write-Result -Item "ネットワークアクセス:ローカルアカウントの共有とセキュリティモデルがクラシックになっているか確認" -IsSuccess (Test-LocalAccountSharingClassic)

#管理共有が有効化されているか
$isBuildInAdministrator = $false
if ($user_account) {
    $isBuildInAdministrator = Test-BuildInAdministrator -sid $user_account.SID
}
Write-Result -Item "管理共有が有効か確認" -IsSuccess (Test-AdminShareEnabled -isBuildInAdministrator $isBuildInAdministrator)

Exit